
      subroutine fehl (f,neqn, y, t, h,
     1                 yp , f1   , f2   , f3,
     2                 f4 , f5 , s)
c     single-step solver for Fehlberg 4-th/5-th order Runge-Kutta O.D.E.

c       program status:                   8 October 1982

c       J. G. Dworski

c       H a n d l i n g :
c       -----------------

c       /fehl (the single-step solver) is called many times by
c       /rkfs (the multi-step solver). The latter is called by
c       /rkf45 (the solution "interfacing" routine),
c       and /rkf45 is called by Mainline.

c       Mainline calls on /rkf45, having formulated and declared
c       "external" the  s i n g l e  subroutine f(t,y,yp) whose
c       name (illustrated by f) is arbitrary, but whose arguments
c       are  e x a c t l y  (t,y,yp), where: /t/ is the (scalar) time,
c       /y/ is the vector of y-values, /yp/ is the vector of dy/dt
c       values, both of size /neqn/. f(t,y,yp) describes the system
c       of /neqn/ simultaneous ODEs.
c       All other pertinent info that f needs to describe the ODE
c       system must be conveyed to it by means of labeled commons.

c       A b s t r a c t

c       /fehl/ integrates  o n e  interval for a system of /neqn/
c       first order O.D.E.s, having form :
c               dy(i)/dt  =  f(t,y(1),y(2),...,y(neqn)) ,
c               where the y(i) and the derivatives yp(i)
c               are given at the starting time, t, of the
c               interval of integration (of size h).
c       It returns the "5-th order" Runge-Kutta approximate solution
c       (locally accurate to 6-th order) at time  t+h.  Solution
c       is stored in the vector s (of size neqn).

c       M e t h o d

c       Runge-Kutta-Fehlberg (4,5) algorithm described in :
c               E. Fehlberg : "Low-order classical Runge-Kutta
c                              formulas with stepsize control",
c                              NASA Tech.Rep. R-315.
c       see also :
c       E. Fehlberg (1969)  : "Klassische Runge-Kutta Formeln
c               fuenfter und siebenter Ordnung mit Schrittweite-
c               Kontrolle",   Computing, Vol.4, pp. 93-106
c                             ---------

c       P a r a m e t e r s

c       f       -- subroutine f(t,y,yp), to evaluate derivatives
c                  yp(i) = dy(i)/dt
c       neqn    -- number of equations to be integrated
c       y(*)    -- solution vector at time /t/
c       t       -- (time), the independent variable of integration
c       h       -- (input) step-size, should be no smaller than
c                          13 times the "machine epsilon" roundoff
c                          in t (to distinguish various arguments)
c       s(*)    -- (output) array (of size /neqn/) containing solution
c                          vector at time  /t+h/

c       w o r k   arguments (for internal use) :
c       five   a r r a y s  of minimal size /neqn/ :
c       / f1, f2, f3, f4, f5/

c       Remark :  the formulas have been "grouped" by use of redundant
c                 parentheses to control loss of significance

c==========================================================================
c     changes:
c       changed "double precision" to "real*8"  (craig friesen 7/8/92)
c==========================================================================

        implicit real*8 (a-h,o-z)
        real*8              y(neqn), t, h, ch
        real*8              yp(neqn),f1(neqn),f2(neqn),f3(neqn)
        real*8              f4(neqn),f5(neqn)
        real*8              s(neqn)
c        integer neqn, k

        external  f

        ch      =  h/4.d0
        do 010 k= 1,neqn
          f5(k) = y(k) + ch*yp(k)
  010   continue

c       ************************************
        call f( t+ ch      ,f5,f1 )
c       ************************************

        ch      = 3.d0*h/32.d0
        do 020 k= 1,neqn
          f5(k) = y(k) + ch*(yp(k) + 3.d0*f1(k))
  020   continue

c       ************************************
        call f( t+ 3.d0*h/8.d0 ,f5,f2 )
c       ************************************

        ch      = h/2197.d0
        do 030 k=1,neqn
          f5(k) = y(k) + ch*(1932.d0*yp(k)
     1                 + (7296.d0*f2(k)-7200.d0*f1(k)))
  030   continue

c       ************************************
        call f( t+12.d0*h/13.d0,f5,f3 )
c       ************************************

        ch      = h/4104.d0
        do 040 k=1,neqn
          f5(k) = y(k) + ch*((8341.d0*yp(k) -   845.d0*f3(k)) +
     1                      (29440.d0*f2(k) - 32832.d0*f1(k))  )
  040   continue

c       ************************************
        call f( t+ h       ,f5,f4 )
c       ************************************

        ch      = h/20520.d0
        do 050 k=1,neqn
          f1(k) = y(k) + ch*((-6080.d0*yp(k) + (9295.d0*f3(k)
     1                        -5643.d0*f4(k))) +
     2                       (41040.d0*f1(k) - 28352.d0*f2(k)))
  050   continue

c       ************************************
        call f( t+ h/2.d0    ,f1,f5 )
c       ************************************


c       compute approximate solution valid at  t+h
c       ------------------------------------------

        ch      = h/7618050.d0
        do 060 k=1,neqn
          s(k) = y(k) + ch*((902880.d0*yp(k) + (3855735.d0*f3(k)
     1                     -1371249.d0*f4(k))) +
     2                     (3953664.d0*f2(k) +   277020.d0*f5(k)))
  060   continue

        return
        end

