
      subroutine rkf45(f,neqn,y,t,tout,relerr,abserr,iflag,work,iwork)
	implicit real*8  (a-h,o-z)
c      "interfacing" subroutine for :
c       Fehlberg 4-th/5-th order Runge-Kutta O.D.E. solver

c       program status:                   10 October 1989

c       J. G. Dworski

c       written by:  H. A. Watts and L. F. Shampine
c                    Sandia Laboratories
c                    Albuquerque, New Mexico

c       /rkf45/ is designed for non-stiff and mildly stiff O.D.E.,
c       assuming that derivative evaluations are not too expensive.
c       It exercises a COMPLETELY DATA ADAPTIVE stepping strategy
c      (the user is not even permitted to suggest an initial step size).
c       It is controlled merely by the specified error criterion and
c       the desired output break-points. Unlike a predictor-corrector,
c       /rkf45 is a "marching" algorithm that practically cannot be
c       stalled by "adverse" data. It reports the level of "adversity",
c       and is automatically parametrized to forge on, if so instructed
c      (see iflag = +4, or +6, or +7).

c       r e m a r k  ---  the FORTRAN code of /rkf45/ is
c                         very short because it serves mainly as
c                         interface between the user and the
c                         actual O.D.E. solver --- the subroutine
c                         /rkfs/;  reason for this architecture
c                         is the long calling sequence of /rkfs/

c       H a n d l i n g :
c       -----------------
c       Order of the ODE (or size of the ODE system) is limited
c       by size of arrays /y,work,iwork/ in Mainline. That size
c       should be .ge. "minsize" shown in "Parameters" list below.

c       Mainline calls on /rkf45, after having formulated and declared
c       as "external" the  s i n g l e  subroutine f(t,y,yp) whose
c       name (illustrated by f) is arbitrary, but whose arguments
c       are  e x a c t l y  (t,y,yp), where: /t/ is the (scalar) "time",
c       and /y/ is the vector of y-values. /yp/ is the vector of dy/dt
c       values (not shown explicitly here but "hidden" w/in array /work/).
c       Both /y and /yp have size /neqn/.
c       f(t,y,yp) describes the system of /neqn/ simultaneous O.D.E.s
c       (more about it below).

c       Also, Mainline has to initialize all /neqn/ elements of the
c       vector /y/ to equal the IC of the given initial value problem.

c       All other information that f may need to describe the O.D.E.
c       system must be conveyed to it by means of labeled commons.

c       Action: /rkf45 calls on /rkfs (the multistep solver), and
c               /rkfs  calls on /fehl (the one-step solver).


c       A b s t r a c t

c       /rkf45/ integrates a system of /neqn/ 1-st order O.D.E.s
c       having form :
c               dy(i)/dt  =  F(t,y(1),y(2),...,y(neqn)) ,
c       where : the y(i) are given at "time" /t/.

c       Typically, /rkf45/ is used to integrate from /t/ to /tout/
c       (see below for the non-trivial looping "syntax"),  but it can
c       be alternatively used as a "one-step integrator" to advance the
c       solution a single step in the direction of /tout/.
c       On any of the two types of "return", all parameters in the calling
c       sequence (except /tout) are automatically set for continuing the
c       integration.  The user has only to call /rkf45/ again
c       - defining a new value for /tout/ - .

c       As stated, /rkf45/ is an "interfacing" routine :
c       it calls the "solution" subroutine /rkfs/. In turn, /rkfs/
c       calls the subroutine /fehl/ which computes an approximate
c       solution over one step.

c       M e t h o d

c       Runge-Kutta-Fehlberg (4,5) algorithm described in :
c               E. Fehlberg : "Low-order classical Runge-Kutta
c                              formulas with stepsize control",
c                              NASA Tech.Rep. R-315.
c       (available at the U.of W. Engr.Libr. : 629.1771 UN32TR)

c       the performance of /rkf45/ is evaluated in :
c       L. F. Shampine, H. A. Watts and S. Davenport :
c               "Solving non-stiff ordinary differential equations,
c                the state of the art",
c                Sandia Laboratories Report  SAND-75-0182, March 1975
c       (available at the U.of W. Engr.Libr. as microfishe, 71 pages),
c       and in the article (same authors and title) in :
c       SIAM Review, Vol. 18, No.3, July 1976, pp 376 - 411
c       (available at the U.of W. Math.Res.Libr. : 510.5 SIA v.18)

c       P a r a m e t e r s

c       f       -- subroutine f(t,y,yp), to evaluate derivatives
c                  yp(i) = dy(i)/dt
c       neqn    -- number of equations to be integrated
c       y(*)    -- solution vector at time /t/, minsize = neqn
c       t       -- (time), the independent variable of integration
c       tout    -- output time (point) at which solution is desired
c       relerr  -- relative error (tolerance) for "local error" test;
c                  for 32 bit/word machines make relerr.ge.1.5E-07
c                  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
c       abserr  -- absolute error (tolerance) for "local error" test
c               note: the code requires (at each step, and for each
c                     component of the "local error" and "solution"
c                     vector) that :
c                     abs("local error") .le. relerr*abs(y) + abserr
c       iflag   -- "status of integration" indicator
c       work (*)-- (internal) "work" array, minsize = 6*neqn + 3
c       iwork(*)-- integer (internal) "work" array,  minsize = 5


c       I n i t i a l i z a t i o n
c       ( 1-st call to /rkf45/ )
c       ========================

c       provide calling program storage for, and in sizes:
c               y(neqn), work(6*neqn+3), iwork(5)
c       declare /f/ in an "external" statement,
c       supply subroutine f(t,y,yp), and
c       initialize parameters :
c       neqn    -- .ge. 1
c       y(*)    -- vector of  i n i t i a l  conditions
c       t       -- starting value of time (must be a FORTR"variable")
c       tout    -- output time (point) at which solution is desired,
c                  tout = t is allowed on  f i r s t  call only, in
c                  which case /rkf45/ returns with  iflag=+2 (if
c                  continuation of integration is possible).
c          remark: to illustrate - say, initial desired time were t0 -
c                  then set t=t0, tout=t and call rkf45; it will
c                  initialize all internal parameters without advancing
c                  permitting you to print-out the initial "state"
c                  before it gets altered. Next, set tout=t+deltat
c                  and this becomes the the first non-trivial loop.
c       relerr  -- rel.local err. tolerance;
c                  for 32 bit/word machines make relerr.ge.1.5E-07
c                  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
c       abserr  -- abs.local err. tolerance; could be set to zero
c                  if one were sure that the solution does not vanish
c                  at any t (when solution vanishes then the relative
c                  error is not defined, and the absolute error tolerance
c                  must be non-zero)
c                  (relerr must be a FORTR"variable",
c                   abserr may  be a FORTR"constant")
c             note: a machine dependent internal rel.error is also
c                   computed;  if /relerr/ is specified smaller than
c                   that, /rkf45/ will increase /relerr/ and return
c                   (iflag=+3); user will have to request continuation
c                   of integration.
c       iflag   --  +1 = normal user initialization of a NEW problem
c                        in the "looping : tout=t+deltat mode"
c                   -1 = single-step-integration mode, relatively slow,
c                        should be invoked when control essential
c                        (such as when iflag=+5 on previous step,
c                         see below, author's comments) or when you
c                         do want /rkf45/ to decide on optimal output
c                         spacing (Dworski's comment).

c       O u t p u t

c       y(*)    -- solution vector at time /t/
c       t       -- last (t-point) value reached by integration

c       iflag   --  +2 = normal return, /tout/ reached by integration
c                   -2 = single step in /tout/ direction executed O.K.
c                        (normal for continuing single-step-integration)

c               --  +3 = c a u t i o n  ---  integration was  n o t
c                        completed, because :
c                        /relerr/ was specified too small; it was
c                        internally increased and /rkf45/ is ready
c                        for (user initiated) continuation
c               --  +4 = c a u t i o n  ---  integration was temporarily
c                        halted, because the user apparently demands
c                        still more output (his output agenda is not
c                        exhausted) even though more than 4000 steps
c                        were executed (more precisely, more than 25000
c                        derivative evaluations were made, with any
c                        number of outputs - or even no output).
c                        Continuation of the run is simply effected by
c                        merely setting iflag to +2 (this also resets the
c                        step counter to zero).
c                        This "caution" condition is typically harmless, and
c                        it may come naturally in jobs that are intrinsically
c                        l o n g , because the step counter is not reset to
c                        zero at any normal output point - for a reason:
c                        to protect a careless user from specifying an
c                        outlandishly large maximum /tout. The user who
c                        wants a long run may continue the solution easily
c                        as stated above, and on each continuation gets
c                        an allowance of about 4000 more steps.
c               --  +5 = w a r n i n g ---  integration was  n o t
c                        completed because :
c                        solution  v a n i s h e d,  making
c                        a   p u r e l y   relative
c                        error test impossible;  use non-zero /abserr/
c                        specification to continue (a good way to
c                        proceed is by single-step-integration mode)
c               --  +6 = w a r n i n g ---  integration was  n o t
c                        completed because:
c                        requested accuracy was not achieved even
c                        with smallest allowable step-size; error
c                        tolerance specification must be increased
c                        before continuing integration
c               --  +7 = w a r n i n g  --- /rkf45/ is inefficient
c                        in solving  t h i s  problem (too much
c                        requested output is restricting the optimal
c                        internal step-size choice);  use the
c                        single-step-integration mode or change
c                        method (ditch /rkf45).
c               --  +8 = e r r o r  return --- invalid  i n p u t
c                        likely causes:   any of following is .true.
c                          (t=tout .and. (iflag.ne.1.or.iflag.ne.-1) )
c                          (relerr.le.0 .or.   abserr.le.0)
c                          (iflag .eq.0,.or. .lt.-2, .or. gt.8)

c       work(*),iwork(*) --  info that is ordinarily of no interest
c                            to the user:
c                        work(i),i=1,...,neqn  contain 1-st derivative
c                                of the solution vector y at (time) t,
c                        work(neqn+1) = h = step-size to be attempted
c                                           on next integration step,
c                        iwork(1) -- derivative evaluation counter, it
c                                    reaches a maximum (25000) and
c                                    /rfk45 pauses with iflag=+4. The
c                                    counter is reset to zero only if
c                                    the user resets iflag to +2 or -2.
c                                    (see iflag = +4, below).


c       ( 2-nd and subsequent calls to /rkf45/ )
c       ========================================

c       User action will depend upon the returned value of /iflag/:

c       iflag = +2 or iflag = -2    --  normal return
c       ---------------------------------------------
c  (a)  If integration reached /tout/ (iflag=+2), the user need only
c       define a new /tout/ by tout=t+deltat, and call /rkf45/ again.
c  (b)  A single-step-integration mode has on normal return: iflag=-2
c       which indicates that a successful single step was accomplished in
c       the direction of the  c u r r e n t  /tout/. Now, the user may
c       want only the single step results without inspecting the result
c       at /tout, or he may want both, or only the latter. When the
c       user wants the result at /taut in the single step mode, then
c       he needs only to watch /iflag. It will tell him that /tout  was
c       actually reached because  rkf45 will not skip that output point,
c       but will merely pause and change iflag to +2. Thereupon, the user
c       must define a new /tout/, and  r e s e t  iflag to iflag=-2 if
c       the single-step-integration mode is to be continued.
c       This protocol offers the user a powerful run strategy of prescribing
c       a mixture of +2 and -2 instructions at any stage, and any number
c       of times.

c       iflag = +3 or iflag = +4   -- incompl. but resumable integration
c       ---------------------------------------------------------------
c       In the iflag=+3 case /relerr/ was internally increased and the
c       integration may continue - recognizing that the error tolerance
c       is still very tight and the computational process is possibly
c       inefficient.
c       In the iflag=+4 case the computational process
c       has been going on for so long (having used a limiting number of
c       derivative evaluations - set presently at 25000) that there
c       might be concern about the causes. The derivative evaluation
c       counter iwork(1) is not reset in the "looping mode" (+2 or -2
c       as iflag values) between calls to rkf45, and a large value might
c       be legitimate in a very long job. However, when reaching 25000
c       derivative evaluations iflag is set to +4, control is returned
c       to the calling program, and integration may continue (for
c       another 25000 derivative evaluations) only if the calling program
c       decides to (by reseting iflag to +2 or -2).

c       iflag = +5 or iflag = +6  --  remedy by user is mandatory
c                                     or the whole  j o b  STOPS
c       ---------------------------------------------------------
c       iflag=+5 occurs only if abserr=0 was specified, a n d  the solution
c       "vanished" at a point, making the relative error test impossible.
c       The user must re-initialize abserr.gt.0 before continuation,
c       else job terminates.
c       The warning implicit in iflag=+6 should be taken seriously
c       - a singularity may be present, or the solution is changing
c       v e r y    rapidly. If user insists on continuation, /abserr/
c       or /relerr/ (or both) must be increased  a n d   iflag must
c       be reset to iflag=+2 or iflag=-2 (the latter for single-step
c       integration), else job terminates.

c       iflag = +7 or iflag = +8  --  modifications needed
c       -------------------------------------------------------------
c       iflag=+7 occurs when /rkf45 "sees" that the user requests
c       outputs about twice per step-size that /rkf45 deems optimal.
c       It vindictively records each such occurrence, and when its
c       cup of chagrin overflows (1000 occurrences - ain't that tolerant
c       enough?) it pauses. If it were a person, /rkf45 would then
c       suggest a change of method (i.e., dropping /rkf45/
c       in favor of - say - Adams codes (DE or STEP by Shampine and
c       Gordon (1975) published in the book :
c       Shampine, L.F. and M.K. Gordon (1975),
c       "Computer solution of ordinary differential equations",
c       W.H. Freeman, San Francisco )).
c       However, in case the
c       user insists on /rkf45/: a change to single-step-integration
c                                mode (reset iflag=-2) is advisable;
c       iflag=+8 requires fixing input error pronto ;
c            if corrective action (to iflag=7 or 8) is not taken,
c            and iflag is not reset ...
c            /rkfs/ will cause the job to   s t o p .

c       N o t e --- do not alter /work/ and /iwork/ in  a n y  way,
c                   they contain info for subsequent integration.

c     ------------------------------------------------------------
c     changes:
c       changed "double precision" to "real*8"  (craig friesen 7/8/92)
c     ------------------------------------------------------------
c=====================================================================

        external  f

        real*8     y(neqn), t,tout, relerr,abserr, work(*)
        integer iwork(5)
        integer k1,k2,k3,k4,k5,k6, k1m

c       compute indices for addressing parts of /work/ array
  010   k1m = 1  + neqn
        k1  = 1  + k1m
        k2  = k1 + neqn
        k3  = k2 + neqn
        k4  = k3 + neqn
        k5  = k4 + neqn
        k6  = k5 + neqn

        call rkfs (f,neqn, y,t,tout, relerr,abserr, iflag,
     1             work(1) ,work(k1m),work(k1),work(k2),work(k3),
     2             work(k4),work(k5) ,work(k6),work(k6+1),
     3             iwork(1),iwork(2),iwork(3),iwork(4),iwork(5))

        return
        end
